#!/usr/bin/python2.1
######################################################################################################
#
# File:		libRotate.py
# Description:	Script implementing several mehtods for generating all fax-files needed
# Author's:	Jan Schierig (Jan.Schierig@t.systems.com),
# Created:	Tue Oct 27 2003
# Modified:     Thu Nov 06 2003
# Language:	Python (External Method for Zope)
#
######################################################################################################


import sys
import os
import StringIO
import traceback
import time
import gzip
import Zope
import string
import re

def rwpopen(COMMAND,ARGS,input):
###################################################################################################
#Method for deadlock-free execution of commandline-tools (workaround for popen2 module)
#  method uses 3 independent processes for writing to stdout, reading from stdin and execution of
#  commandline-tool (reads stdin and writes to stdout)
#
#parameter COMMAND is commandline for call
#parameter ARGS is a list of commandline arguments
#
###################################################################################################


  def safe_traceback():
    sys.stderr.write("Error in child %d.\n" % os.getpid())
    sys.stderr.flush()
    traceback.print_exc()
    sys.stderr.flush()



  p_readfd, c_writefd = os.pipe()
  c_readfd, p_writefd = os.pipe()


  if os.fork():
    #Parent-Process
    for fd in (c_writefd, c_readfd, p_writefd):
      os.close(fd)


    fp = os.fdopen(p_readfd,'r')
    result = fp.read()
    os.wait()
    fp.close()
    return result

  else:
    #Child-Process
  
    try: 
      if os.fork():

        for x in input.readlines():
          os.write(p_writefd, x)
        #os.write(p_writefd, input.read())
      

      else:
        #Grandchild-Process
        try: 
          os.close(0)
          os.dup(c_readfd)

          os.close(1)
          os.dup(c_writefd)

	  for fd in (c_writefd, c_readfd, p_writefd, p_readfd):
            os.close(fd)
          
          os.execvp(COMMAND,ARGS)
          os.wait()

        except:
          safe_traceback()
          os._exit(127)
  
    except:
      safe_traceback()
      os._exit(127)

    else:
      os._exit(0)




def genGZ(self,file,ctx):

#############################################################################################################
#Method for generating GZIP-archive, that contains a fax in postscript-format - gz is written back into ZODB
#
#parameter file is Zope-Object (tif-File)
#parameter ctx is Zope-Context
#
#returm value is ps-Stream (string)
#
#############################################################################################################


  try:
    (tempPath, tempName) = myTmpPath()
    #tempName = str(time.time()) + '.tif'
    #tempPath = 'faxTemp/' + tempName
    tifFile = StringIO.StringIO()
    tifFile.write(str(file.data))
    tifFile.seek(0)

    #generate temporary file for conversion tiff2ps
    tempFile = open(tempPath,'w')
    tempFile.write(tifFile.read())
    tifFile.close()
    tempFile.close()
  except:
    pass


  #executionn of tiff2ps without Zombie-Process
  
  TIFF2PS = self.TIFF2PS_CMD + ' ' + tempPath
  output = os.popen(TIFF2PS,'r')
  psFile = StringIO.StringIO()
  psFile.write(output.read())
  psFile.seek(0)
  output.close()

  ps = open('faxTemp/ps.ps','w')
  ps.write(psFile.read())
  ps.close()
  psFile.seek(0)


  #delete temporary file
  DEL = 'rm ' + tempPath
  os.system(DEL)

  gzBuf = StringIO.StringIO()
  gzFile = gzip.GzipFile(mode='w', fileobj = gzBuf, compresslevel = 9)
  gzFile.write(psFile.read())
  gzFile.close()

  psFile.seek(0)
  returnVal = psFile.read()


  #set name for ps.gz-file
  name = string.split((file.getId()),'.')[0] + '.ps.gz'

  ctx.manage_addProduct['OFSP'].manage_addFile(id=name, file=gzBuf, content_type='application/x-gzip')
  gzBuf.close()
  psFile.close() 


  return returnVal





def genPNG(self,file,ctx):
###################################################################################################
#Method for generating a number of PNGs (one for each fax-sheet) - PNGs are written back into ZODB
#
#parameter file is Zope-Object (tif-File)
#parameter ctx is Zope-Context
#
###################################################################################################

  (tempPath, tempName) = myTmpPath()

  tifFile = StringIO.StringIO()
  tifFile.write(str(file.data))
  tifFile.seek(0)

  #generate temporary file for conversion tiff2ps
  tempFile = open(tempPath,'w')
  tempFile.write(tifFile.read())
  tifFile.close()
  tempFile.close()

  objName = string.split(tempName, '.')[0]

  try:
    TIFFSPLIT = self.TIFFSLIT_CMD + ' ' + tempPath + ' faxTemp/' + objName
    splitOut = os.system(TIFFSPLIT)

    ##### Delete Temp-File #####

    try:
       DEL = 'rm ' + tempPath
       os.system(DEL)
    except:
      return '<P ALIGN=CENTER><strong>Delete Temp-File ... Failed !</strong></P>'

    ##### Work with splited Tiff-Files #####

    itemFolder = 'faxTemp/'
    itemList   = []
    itemList   = os.listdir(itemFolder)


    c=1

    for tiffs in itemList:
      o = re.search(objName, tiffs)

      if o != None:
        try:
          TIFF2PNG = self.TIFF2PNG_CMD + ' faxTemp/ faxTemp/' + tiffs
          pngOut = os.system(TIFF2PNG)
        except:
          return '<P ALIGN=CENTER><strong>Generate a PNG-Object ... Failed !</strong></P>'
    
        try:
          DEL = 'rm faxTemp/' + tiffs
          os.system(DEL)
        except:
          return '<P ALIGN=CENTER><strong>Delete TIFF-Objects ... Failed !</strong></P>'


        try:
          pngTemp = 'faxTemp/' + string.split(tiffs, '.')[0] + '.png'
          device = open(pngTemp,'r')
          pngFile = StringIO.StringIO()
          pngFile.write(device.read())
          device.close()
          pngFile.seek(0)
          pngName = string.split(str(file.getId()), '.')[0] + '.' + str(c) + '.png'
          ctx.manage_addProduct['OFSP'].manage_addFile(id=pngName, file=pngFile, content_type='image/png')
          pngFile.close()
        except:
          return '<P ALIGN=CENTER><strong>Put PNG-Files in ZOPE-DB ... Failed !</strong></P>'

 
        try:
          DEL = 'rm ' + pngTemp
          os.system(DEL)
        except:
          return '<P ALIGN=CENTER><strong>Delete PNG-Files ... Failed !</P>'

        c=c+1

  except:
    return '<P ALIGN=CENTER><strong>Create PNG-Files ... Failed !'





def myTmpPath():
###################################################################################################
#Method for generating a temporary filename myName in path /usr/local/Zope/var/faxTemp 
#
#return value is a list of 2 elements
#  1st element: relative path of temporary file faxtemp/<temp. filename>
#  2nd element: name of the temporary file
#
###################################################################################################
  myName = str(time.time()) + '.tif'
  tmpPth = 'faxTemp/' + myName

  return [tmpPth, myName]
